//
// C++ Implementation: gui
//
// Description: Graphical User Interface, and image computation for mulot.
//
//
//     Copyright (C) 2008  Romain Fontugne <romain.fontugne@gmail.com>
//
//     This program is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     This program is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
#include "gui.h"

#include <map> //todo tmp
/**
 *	packet analyzer callback routine, called from the sniffer
 */
void readDump(u_char* aUserPtr, const pcap_pkthdr* aHdrPtr, const u_char *aDataPtr){
    const u_char*	nextHdr;
    EtherHeader*	etherHdr = (EtherHeader*) aDataPtr;
    IcmpHeader*		icmpHdr	= NULL;
    IpHeader*		ipHdr	= NULL;
    TcpHeader*		tcpHdr	= NULL;
    UdpHeader*		udpHdr	= NULL;

    u_short		etherType = ntohs(etherHdr->etherType);
//    IcmpType		icmpInfo;
//    u_char		icmpCode;
    InetAddr 		srcAddr;
    InetAddr 		destAddr;
    int32_t		srcPort;
    int32_t		destPort;
    int			etherPktLen;
    int			etherHdrLen;
    int			ipPktLen;
    int			ipHdrLen;
    int			transHdrLen;
//    char		String = 'o';
    int			x;			// x coordinate
    int			y;			// y coordinate
    double		tmp;
    GUI			*gui = (GUI*)aUserPtr;
    pkt			p;

    static int n[65535];
    static int m[65535];
    static int sumn;
    static int summ;

    if (etherType != ETHERTYPE_IP)
	return;

    if (isEtherBroadcast(etherHdr->etherDest)) 
	return;

    if (gui->firstTick == 0){
	gui->firstTick = aHdrPtr->ts.tv_sec;
	gui->startTick = aHdrPtr->ts.tv_sec + gui->mouseX * (gui->timeRange / (double)gui->menuX);
    }
    else if (aHdrPtr->ts.tv_sec < gui->startTick)
	return;

    etherPktLen = aHdrPtr->len;
    etherHdrLen = ETHER_HDR_LEN;

    ipHdr = (IpHeader *) (aDataPtr + etherHdrLen);
    srcAddr = ipHdr->ipSrc;
    destAddr = ipHdr->ipDest;
    ipPktLen = ipHdr->ipLength;
    ipHdrLen = ipHdr->ipHeaderLength*4;

    if (isIpMulticast(destAddr))
	return;

    nextHdr = aDataPtr + etherHdrLen + ipHdrLen;
    switch (ipHdr->ipProto) {
	case IPPROTO_ICMP:
	    icmpHdr = (IcmpHeader *) nextHdr;
	    //icmpInfo = icmpHdr->icmpInfo;
	    //icmpCode = icmpHdr->icmpCode;
	    destPort = -1;
	    srcPort  = -1;
	    break;
	case IPPROTO_TCP:
	    tcpHdr = (TcpHeader *) nextHdr;
	    srcPort = ntohs(tcpHdr->tcpSrcPort);
	    destPort = ntohs(tcpHdr->tcpDestPort);
	    transHdrLen = tcpHdr->tcpOffset;
	    break;
	case IPPROTO_UDP:
	    udpHdr = (UdpHeader *) nextHdr;
	    srcPort = ntohs(udpHdr->udpSrcPort);
	    destPort = ntohs(udpHdr->udpDestPort);
	    transHdrLen = ntohs(udpHdr->udpLength);
	    break;
	default :
	    srcPort = -1;
	    destPort = -1;
    }

    // Compute x
    x = ((((double)aHdrPtr->ts.tv_sec+(double)aHdrPtr->ts.tv_usec/1000000.) - (double)gui->startTick))/ gui->timeCoef;
    // Check if x is in the picture
    if (x >= gui->mainImg[gui->yMode[0]-'a'].dimx() || (gui->loopMode=='d' && x > gui->glassX + gui->glassSize)){
 	gui->sniff->breakloop();
	return;
    }
    else if(x < 0 || (gui->loopMode=='d' && x < gui->glassX - gui->glassSize))
	return;

	// store the offset and the file ind
    #ifndef __MINGW32__
    if(!gui->seekComputed){
	int xM = (x * gui->menuX/gui->mainX) + 1;
	if(gui->seek[xM].fileInd == -1){ 
		gui->seek[xM].fileInd = gui->sniff->fileInd; 
		gui->seek[xM].offset = ftell(pcap_file(gui->sniff->currPcapDesc));
	}
    }
    #endif
    // Compute y
    switch (gui->yMode[0]){
	case 'a' :
		tmp = destAddr & (0xffffffff >> gui->cidrDest);
		tmp -= gui->startY[0];
		tmp /= gui->yCoef[0];
		y = tmp;
		break;
	case 'b' :
    		y = (destPort - gui->startY[1])/gui->yCoef[1];
		if(y >= gui->mainImg[gui->yMode[0]-'a'].dimy() || y < 0) return;
		break;
	case 'c' :
		tmp = srcAddr & (0xffffffff >> gui->cidrSrc);
		tmp -= gui->startY[2];
		tmp /= gui->yCoef[2];
		y = tmp;
		break;
	case 'd' :
		y = (srcPort - gui->startY[3])/gui->yCoef[3];
		if(y >= gui->mainImg[gui->yMode[0]-'a'].dimy() || y < 0) return;
		break;
	default :
		y=0;
    }

    if(y >= gui->mainImg[gui->yMode[0]-'a'].dimy() || y < 0 ) return;

    if(gui->loopMode == 'i'){
//switch(gui->yMode){
	//y = 0;
	//	case  'e':
	switch (ipHdr->ipProto) {
	case IPPROTO_TCP:	(*gui->mainStat)(x,0) ++; if((*gui->mainStat)(x,0)>gui->yRange[4]) gui->yRange[4] = (*gui->mainStat)(x,0); break;
	case IPPROTO_UDP:	(*gui->mainStat)(x,1) ++; if((*gui->mainStat)(x,1)>gui->yRange[4]) gui->yRange[4] = (*gui->mainStat)(x,1); break;
	case IPPROTO_ICMP:	(*gui->mainStat)(x,2) ++; if((*gui->mainStat)(x,2)>gui->yRange[4]) gui->yRange[4] = (*gui->mainStat)(x,2); break;
	default :		(*gui->mainStat)(x,3) ++; if((*gui->mainStat)(x,3)>gui->yRange[4]) gui->yRange[4] = (*gui->mainStat)(x,3);
	}
	//break;
	//	case 'f':
	(*gui->mainStat)(x,4) += aHdrPtr->len;		//  length in bytes
	if((*gui->mainStat)(x,4)>gui->yRange[5]) gui->yRange[5] = (*gui->mainStat)(x,4);
	//break;
	//	case 'g':
	(*gui->mainStat)(x,5) += aHdrPtr->len;		//  length in bytes
	(*gui->mainStat)(x,6)++;
	//	case 'h':
	if(ipHdr->ipProto==IPPROTO_TCP){
// 		if(tcpHdr->tcpFlags & TH_SYN)	(*gui->mainStat)(x,7) ++;
// 		if(tcpHdr->tcpFlags & TH_ACK)	(*gui->mainStat)(x,8) ++;
// 		if(tcpHdr->tcpFlags & TH_FIN)	(*gui->mainStat)(x,9) ++;
// 		if(tcpHdr->tcpFlags & TH_RST)	(*gui->mainStat)(x,10) ++;
// 		if(tcpHdr->tcpFlags & TH_PUSH)	(*gui->mainStat)(x,11) ++;
// 		if(tcpHdr->tcpFlags & TH_URG)	(*gui->mainStat)(x,12) ++;
// 		if(tcpHdr->tcpFlags & TH_ECE)	(*gui->mainStat)(x,13) ++;
// 		if(tcpHdr->tcpFlags & TH_CWR)	(*gui->mainStat)(x,14) ++;
	}

	static map<int,char> diffHdrS;
	static map<int,char> diffHdrD;
	if(ipHdr->ipProto==IPPROTO_TCP || ipHdr->ipProto==IPPROTO_UDP){
		if((*gui->mainStat)(x,9) == 255){
			//compute last entropy and set that one to 0
			(*gui->mainStat)(x,9) = 0;
			(*gui->mainStat)(x,10) = 0;
			if(x>0){	
				for(int i = 0; i<65535; i++){
					if(n[i]!=0)	(*gui->mainStat)(x-1,9) += 1000 * ((double)n[i]/(double)sumn) * log2((double)n[i]/(double)sumn);
					if(m[i]!=0)	(*gui->mainStat)(x-1,10) += 1000 * ((double)m[i]/(double)summ) * log2((double)m[i]/(double)summ);
					n[i]	= 0;
					m[i]	= 0;
				}

				(*gui->mainStat)(x-1,9) = -(*gui->mainStat)(x-1,9);
				(*gui->mainStat)(x-1,10)= -(*gui->mainStat)(x-1,10);
				
				sumn = 0;
				summ = 0;

				// Nb of different features
//  				for(int i = 0; i<65535; i++){ 	if(n[i]!=0) (*gui->mainStat)(x-1,9)++;	n[i] = 0;
// 								if(m[i]!=0) (*gui->mainStat)(x-1,10)++;	m[i] = 0;}
// 				(*gui->mainStat)(x-1,12) = diffHdrS.size();
// 				(*gui->mainStat)(x-1,11) = diffHdrD.size();
// 				diffHdrD.clear();
// 				diffHdrS.clear();
			}
		}
// 		int tmp;
//  		tmp = (((int)destAddr) << 16) | destPort ;
// 		diffHdrD.insert(pair<double,char>(tmp,'t'));
//  		tmp =  srcPort ;
// 		diffHdrS.insert(pair<int,char>(tmp,'t'));
		n[destAddr & 0x0000ffff]++;
		sumn++;
		m[srcAddr & 0x0000ffff]++;
		summ++;

// 		else{	// nb port used
// 			if(n[destPort]==0){ n[destPort]++; sumn++;}
// 			
// 			if(m[srcPort]==0){ m[srcPort]++; summ++;}
// 		}
	}
	//break;
//	default :
	for(char mode='a'; mode<='d'; mode++){
		// Compute y
		switch (mode){
			case 'a' :
				tmp = destAddr & (0xffffffff >> gui->cidrDest);
				tmp -= gui->startY[0];
				tmp /= gui->yCoef[0];
				y = tmp;
				break;
			case 'b' :
				y = (destPort - gui->startY[1])/gui->yCoef[1];
				if(y >= gui->mainImg[1].dimy() || y < 0) continue;
				if(gui->mainImg[1](x,(gui->mainImg[1].dimy()-1) - y,0,2)==255) (*gui->mainStat)(x,7) ++;
				break;
			case 'c' :
				tmp = srcAddr & (0xffffffff >> gui->cidrSrc);
				tmp -= gui->startY[2];
				tmp /= gui->yCoef[2];
				y = tmp;
				break;
			case 'd' :
				y = (srcPort - gui->startY[3])/gui->yCoef[3];
				if(y >= gui->mainImg[3].dimy() || y < 0) continue;
				if(gui->mainImg[3](x,(gui->mainImg[3].dimy()-1) - y,0,2)==255) (*gui->mainStat)(x,8) ++;
				break;
			default :
				y=0;
		}
		if(y >= gui->mainImg[mode-'a'].dimy() || y < 0) continue;
		y = (gui->mainImg[mode-'a'].dimy()-1) - y;	// Reverse order to have minimum value at the bottom
		if(gui->colorMode == 'c'){
				//!if(gui->nbAnnotation && gui->mainImg[mode-'a'](x,y,0,0) == 0)
					//;						// nothing the pixel is black
				//!else if(gui->nbAnnotation && gui->annotation->validate((pcap_pkthdr*)aHdrPtr,aDataPtr)){		// anomaly are shown in black
				//!	gui->mainImg[mode-'a'](x,y,0,0) = 0;
				//!	gui->mainImg[mode-'a'](x,y,0,2) = 0;
				//!	gui->mainImg[mode-'a'](x,y,0,1) = 0;
				//!}
				//!else 
				if(gui->mainImg[mode-'a'](x,y,0,2) < 255){		// coloring pixel
					if(gui->mainImg[mode-'a'](x,y,0,2)>0){	
						gui->mainImg[mode-'a'](x,y,0,0) -= 2;		// remove red
						gui->mainImg[mode-'a'](x,y,0,2) -= 5;		// remove blue
						gui->mainImg[mode-'a'](x,y,0,1) -= 1;
					}else if(gui->mainImg[mode-'a'](x,y,0,0)<255){	
						gui->mainImg[mode-'a'](x,y,0,0) += 3;		// add red
						//gui->mainImg[mode-'a'](x,y,0,1) -= 1;
					}else if(gui->mainImg[mode-'a'](x,y,0,1)>0){
						gui->mainImg[mode-'a'](x,y,0,1) -= 1;		// remove green
					}
				}
				else{
					gui->mainImg[mode-'a'](x,y,0,0) = 150;
					gui->mainImg[mode-'a'](x,y,0,1) = 210;
					gui->mainImg[mode-'a'](x,y,0,2) = 150;
				}
		}
		else{	// monocolor
			if(gui->mainImg[mode-'a'](x,y,0,2) == 255){
				gui->mainImg[mode-'a'](x,y,0,0) = 50;
				gui->mainImg[mode-'a'](x,y,0,1) = 50;
				gui->mainImg[mode-'a'](x,y,0,2) = 50;
			}
			else if(gui->mainImg[mode-'a'](x,y,0,2) > 0){
				gui->mainImg[mode-'a'](x,y,0,0) --;
				gui->mainImg[mode-'a'](x,y,0,1) --;
				gui->mainImg[mode-'a'](x,y,0,2) --;
			}
		}
	}
	gui->count++;
	if(gui->mainDisp && gui->count>1000000){
		gui->displayRefresh(-1);
		gui->count = 0;
	}
	//}
    }
    else if(gui->loopMode == 'a'){	/***** Compute pictures for anomaly detection *****/
	int subNetwork = 0;
	// Keep packet data
	p.ts = aHdrPtr->ts;
	p.srcAddr = ipHdr->ipSrc;
	p.destAddr = ipHdr->ipDest;
	p.proto	= ipHdr->ipProto;
	p.len = aHdrPtr->len;
	switch (ipHdr->ipProto) {
		case IPPROTO_TCP:	p.hdr.tcp = new TcpHeader; *p.hdr.tcp = *tcpHdr; break;
		case IPPROTO_UDP:	p.hdr.udp = new UdpHeader; *p.hdr.udp = *udpHdr; break;
		case IPPROTO_ICMP:	p.hdr.icmp = new IcmpHeader; *p.hdr.icmp = *icmpHdr; break;
		default :		;
	}
	gui->pktList.push_back(p);

	for(int i=0; i<2; i++){
		
		if(i==0){ // Compute pictures for source
			tmp = (srcAddr & (0xffffffff >> gui->cidrSrc))*NB_SUB_DETEC;
			tmp -= gui->startY[2];
			tmp /= gui->yCoef[2];
	//		y = tmp % gui->mainY;
			subNetwork = (tmp-1) / gui->mainY;				//TODO verifier que subNetwork/y ne soit pas trop grand ou petit
		}
		else{ 	// Compute pictures for destination
			tmp = (destAddr & (0xffffffff >> gui->cidrDest))*NB_SUB_DETEC;
			tmp -= gui->startY[0];
			tmp /= gui->yCoef[0];
			//y = tmp;
			subNetwork = (tmp-1) / gui->mainY;				//TODO verifier que subNetwork/y ne soit pas trop grand ou petit
		}
		for(char mode='a'; mode<='d'; mode++){
			// Compute y
			switch (mode){
				case 'a' :
					tmp = (destAddr & (0xffffffff >> gui->cidrDest))*NB_SUB_DETEC;
					tmp -= gui->startY[0];
					tmp /= gui->yCoef[0];
					y = (int)(tmp-1) % gui->mainY;
					break;
				case 'b' :
					y = (destPort - gui->startY[1])/gui->yCoef[1];
					break;
				case 'c' :
					tmp = (srcAddr & (0xffffffff >> gui->cidrSrc))*NB_SUB_DETEC;
					tmp -= gui->startY[2];
					tmp /= gui->yCoef[2];
					y = (int)(tmp-1) % gui->mainY;
					break;
				case 'd' :
					y = (srcPort - gui->startY[3])/gui->yCoef[3];
					break;
				default :
					y=0;
			}
			if(y >= gui->mainY || y < 0) continue;
			y = (gui->detecImg[i][subNetwork][mode-'a'].dimy()-1) - y;	// Reverse order to have minimum value at the bottom
			// Draw point
			if(gui->detecImg[i][subNetwork][mode-'a'](x,y,0,0) == 255){
				gui->detecImg[i][subNetwork][mode-'a'](x,y,0,0) = 0;
			}
			else if(gui->detecImg[i][subNetwork][mode-'a'](x,y,0,0) > 0){
				gui->detecImg[i][subNetwork][mode-'a'](x,y,0,0) --;
			}

			gui->infoDetec[i][subNetwork][mode-'a'][y].insert(pair<short int,unsigned int>((short int)x,gui->pktList.size()-1));
		}
	}
    }
    else if(gui->loopMode == 'd'){	// Store data
	y = (gui->mainImg[gui->yMode[0]-'a'].dimy()-1) - y;	// Reverse order to have minimum value at the bottom
	if (y > gui->glassY+gui->glassSize || y < gui->glassY-gui->glassSize) return;
	p.ts = aHdrPtr->ts;
	p.srcAddr = ipHdr->ipSrc;
	p.destAddr = ipHdr->ipDest;
	p.proto	= ipHdr->ipProto;
	p.len = aHdrPtr->len;
	switch (ipHdr->ipProto) {
		case IPPROTO_TCP:	p.hdr.tcp = new TcpHeader; *p.hdr.tcp = *tcpHdr; break;
		case IPPROTO_UDP:	p.hdr.udp = new UdpHeader; *p.hdr.udp = *udpHdr; break;
		case IPPROTO_ICMP:	p.hdr.icmp = new IcmpHeader; *p.hdr.icmp = *icmpHdr; break;
		default :		;
	}
	gui->info[y].insert(pair<int,pkt>(x,p));
    }
    else{
	cout << "Error! GUI::loopMode = " << gui->loopMode << endl;
    }
}

/**
 *	format an integer as ip address, and mask non-significant bytes
 *	
 */
void printIp(unsigned int n, int bitSignif,char* str){
   struct in_addr x; 
   int j;
   x.s_addr = htonl(n);
   strcpy(str, inet_ntoa(x));			//format as an IP address
   j = 0;
   for(int i = 0; i<floor(bitSignif/8.0); i++){	//mask unknown bytes
	while (str[j] != '.'){
		str[j++] = 'x';
		if(i==3)break;
	}
	j++;
   }
}


GUI::GUI()
{
    silent	= false;
    saveRulers	= true;
    glassX 	= -1;
    glassY 	= -1;
    glassSize 	= 7;
    refreshMenu = true;

    colorMode	= 'c';
    red  	= new unsigned char[3];red[0] = 255;red[1] = 0;red[2] = 0;	// Colors
    blue 	= new unsigned char[3];blue[0] = 0;blue[1] = 0;blue[2] = 255;	
    green 	= new unsigned char[3];green[0] = 0;green[1] = 255;green[2] = 0;	
    white	= new unsigned char[3];white[0] = 255;white[1] = 255;white[2] = 255;
    black	= new unsigned char[3];black[0] = 0;black[1] = 0;black[2] = 0;
    purple 	= new unsigned char[3];purple[0] = 255;purple[1] = 0;purple[2] = 255;	
    orange 	= new unsigned char[3];orange[0] = 255;orange[1] = 200;orange[2] = 0;	
    last 	= new unsigned char[3];last[0] = 0;last[1] = 255;last[2] = 255;	
    grey 	= new unsigned char[3];grey[0] = 127;grey[1] = 127;grey[2] = 127;	
    whiteBg	= 255;

    menuX 	= 420;
    menuY 	= 230;
    mainX 	= CImgDisplay::screen_dimx()-menuX-20;
    mainY 	= (CImgDisplay::screen_dimy()-90)/2;
    nbDisp	= 2;	// nbDisp should be inferior than NB_YMODE

    cidrDest	= 0;
    cidrSrc	= 0;
    timeRange 	= 86400;		// 86400 = 24hours

    yMode	= new char[nbDisp];

    loopMode 	= 'i';		// image (i), data (d), anomaly detection (a)

    startTick 	= 0;		// display after this tick
    for(int i=0; i<NB_YMODE; i++){ startY[i]= 0; yCoef0[i] = 1; yRange[i]=1;}
    count 	= 0;

    // DETECT
    cleanFreq = -1;	//TODO move in detector?
    //!annotation = (void*) NULL;
    nbAnnotation = 0;
}

GUI::~GUI()
{
}

void GUI::init(bool disp)
{
    yRange[0] 	= (0xffffffff >> cidrDest);	// if gBitSignif == 18 then yRange = 16384
    yRange[2]	= (0xffffffff >> cidrSrc);
    yRange[1]	= yRange[3]	= 65536;

    timeCoef0	= (double)timeRange/(double)mainX;	// Coef. so we can see timeRange seconds in the initial window
    yCoef0[0]	= (double)yRange[0]/(double)mainY;	// Coef. so we can see yRange host in the initial window
    yCoef0[2]	= (double)yRange[2]/(double)mainY;	// Coef. so we can see yRange host in the initial window
    yCoef0[1]	= yCoef0[3] = (double)yRange[1]/(double)mainY;

    timeCoefI	= timeCoef = timeCoef0;		// Coef. for the window (x axis)
    //yCoefI	= yCoef[0] = yCoef0[0];	// Coef. for the window (y axis)

    info	= new info_type[mainY];
    seek	= new spot[mainX];
    for (int i = 0; i<mainX ; i++){seek[i].fileInd = -1;seek[i].offset = 0;}
    seekComputed = false;

    mainList	= new CImgList<unsigned char>[nbDisp]();
    mainList2	= new CImgList<unsigned char>[nbDisp]();
    menuList	= new CImgList<unsigned char>();
    btnList	= new CImgList<unsigned char>();	// Define the menu's header
    mapImg	= new CImg<unsigned char>[NB_YMODE];		// Define the menu picture
    infoImg	= new CImg<unsigned char>(menuX,11,1,3,0);		// Define the menu's info
    rulerXImg	= new CImg<unsigned char>(mainX,15,1,3,0);		// Define x axis rule
    rulerYImg	= new CImg<unsigned char>(mainY,15,1,3,0);		// Define y axis rule
    mainImg	= new CImg<unsigned char>[NB_YMODE];		// Define main pictures
    mainStat	= new CImg<unsigned int>(mainX,15,1,1,0);
//    detecImg	= new CImg<unsigned char>[2][NB_SUB_DETEC][NB_YMODE_DETEC];

    if(disp){
	CImg<unsigned char> xZoomIn("img/tIn.png");
	CImg<unsigned char> xZoomOut("img/tOut.png");
	CImg<unsigned char> yZoomIn("img/aIn.png");
	CImg<unsigned char> yZoomOut("img/aOut.png");
	CImg<unsigned char> navL("img/navL.png");
	CImg<unsigned char> navR("img/navR.png");
	CImg<unsigned char> navU("img/navU.png");
	CImg<unsigned char> navD("img/navD.png");
	CImg<unsigned char> home("img/home.png");
	CImg<unsigned char> mode("img/mode.png");
	CImg<unsigned char> mode2("img/mode2.png");
	CImg<unsigned char> save("img/save.png");
	CImg<unsigned char> help("img/help.png");
	btnList->insert(xZoomIn);
	btnList->insert(xZoomOut);
	btnList->insert(yZoomIn);
	btnList->insert(yZoomOut);
	btnList->insert(navL);
	btnList->insert(navR);
	btnList->insert(navU);
	btnList->insert(navD);
	btnList->insert(home);
	btnList->insert(mode);
	btnList->insert(mode2);
	btnList->insert(save);
	btnList->insert(help);
    }

    menuList->insert(btnList->get_append('x'));
    delete btnList;
    for(int i=0; i<NB_YMODE; i++){ 
	mainImg[i].assign(mainX,mainY,1,3,0);
	mapImg[i] = mainImg[i].get_resize(menuX,menuY,1,3);
	yCoef[i]	= yCoef0[i];
    	mouseY[i]	= 0;
    	mouseYR[i]	= 0;
    }
    for(int i=0; i<NB_YMODE_DETEC; i++)
	for(int j=0; j<NB_SUB_DETEC; j++){
		detecImg[0][j][i].assign(mainX,mainY,1,1,0);
		detecImg[1][j][i].assign(mainX,mainY,1,1,0);
	}

    mouseX	= 0;
    firstTick	= 0;
    //Header

    for(int i=0; i<nbDisp; i++){
	mainList2[i].insert(rulerYImg->get_rotate(270));
	mainList2[i].insert(mainImg[i]);
	mainList[i].insert(mainList2[i].get_append());
	mainList[i].insert(*rulerXImg);
	yMode[i] = 'a'+(char)i;
    }
    infoImg->draw_text("Welcome to mulot!",0,0,white);
    menuList->insert(mapImg[0],1,false);
    menuList->insert(*infoImg);
    if(disp){
	menuDisp = new CImgDisplay(menuList->get_append('y','p'),"Menu");           // Display images
    #ifdef __MINGW32__
	menuDisp->move(0,30);
	mainDisp = new CImgDisplay[nbDisp];
	for(int i=nbDisp-1; i>=0; i--){
		mainDisp[i].assign(mainList[i].get_append('y','n'),"Main");
		mainDisp[i].move(menuX+10,mainY*i+(i+1)*30);
	}
    #else
    	menuDisp->move(0,0);
    	mainDisp = new CImgDisplay[nbDisp];
	for(int i=nbDisp-1; i>=0; i--){
		mainDisp[i].assign(mainList[i].get_append('y','n'),"Main");
		mainDisp[i].move(menuX+10,mainY*i+i*30);
	}
    #endif
	display();
	seekComputed = true;
	for(int i=0; i<NB_YMODE; i++)
		mapImg[i] = mainImg[i].get_resize(menuX,menuY,1,3);
	(*menuList)[1] = mapImg[0];
	menuDisp->display(*menuList,'y','p');
    }
    else{
	mainDisp = NULL;
	menuDisp = NULL;
    }
}


/**
 * Refresh display windows for visualization
 * @param disp Number of the display to refresh. -1 for all of them.
 */
void GUI::displayRefresh(int disp){
    if(disp == -1) for(int i=0; i<nbDisp; i++) displayRefresh(i);
    else {
	mainList2[disp][1] = mainImg[yMode[disp]-'a'];

	// Display rulers
	rulersDisplay(yMode[disp],disp);
	if(menuDisp && disp==0){	// Refresh menu
		
		(*menuList)[1] = mapImg[yMode[disp]-'a'];
		menuDisp->resize(menuList->get_append('y','p'));
		// Show on the map the zoomed zone (blue rectangle)
		(*menuList)[1] = mapImg[yMode[disp]-'a'];
		(*menuList)[1].draw_rectangle(mouseX,mouseYR[yMode[disp]-'a'],mouseX+(timeCoef / timeCoef0)*(double)menuX,mouseYR[yMode[disp]-'a']+(yCoef[yMode[disp]-'a']/yCoef0[yMode[disp]-'a'])*(double)menuY,blue,0.3f);
		
		menuDisp->display(*menuList,'y','p');
	}
	if(mainDisp){
		mainDisp[disp].display(mainList[disp],'y','n');
		switch (yMode[disp]){
			case 'a' : 
				mainDisp[disp].set_title("Destination IP address / Time");
				break;
			case 'b' : 
				mainDisp[disp].set_title("Destination port / Time");
				break;
			case 'c' : 
				mainDisp[disp].set_title("Source IP address / Time");
				break;
			case 'd' : 
				mainDisp[disp].set_title("Source port / Time");
				break;
			case 'e' : 
				mainDisp[disp].set_title("Number of packet / Time");
				break;
			case 'f' : 
				mainDisp[disp].set_title("Rate (kbit/s)");
				break;
			case 'g' : 
				mainDisp[disp].set_title("Average packet size (byte) / Time");
				break;
			case 'h' : 
				mainDisp[disp].set_title("Entropy (port) / Time");
				break;
			case 'i' : 
				mainDisp[disp].set_title("Test / Time");
				break;
		}
	}
	
    }
}

void GUI::display(char lMode, int x){
    int  offset = 0;
    char fileInd= 0;
    x= x==-1?	mouseX	:x;
    loopMode	= lMode;
    count 	= 0;
    if(seekComputed && x>=0 && x<menuX && seek[x].fileInd != -1){
	offset  = seek[x].offset;
	fileInd = seek[x].fileInd;
    }
    if (firstTick != 0 && mouseX > 0)
	startTick = firstTick + mouseX * (timeRange / (double)menuX);
    else startTick = firstTick;

    startY[yMode[0]-'a']= mouseY[yMode[0]-'a'] * (yRange[yMode[0]-'a'] / (double)menuY);
    CImg<unsigned char> tmp = (*menuList)[2];
    infoDisplay("loading...",0,false,menuX-50,0);
    if(loopMode == 'i'){
	for(int i=0; i<NB_YMODE; i++) mainImg[i].fill(whiteBg);
	mainStat->fill(0);
	for(int i=0; i<mainStat->dimx(); i++)	(*mainStat)(i,9) = 255; //TODO this is for entropy to remove if entropy is not used
	yRange[4] = 1;
	yRange[5] = 1;
	yRange[6] = 1;
    }
    else if(loopMode == 'd'){
	for (int i = 0; i<mainY ; i++)	info[i].clear();
    }
    else if(loopMode == 'a'){
	pktList.clear();
	for(int i=0; i<NB_YMODE_DETEC; i++)
		for(int j=0; j<NB_SUB_DETEC; j++){
			detecImg[0][j][i].fill(whiteBg);
			detecImg[1][j][i].fill(whiteBg);
			for(int k=0; k<700; k++){
				infoDetec[0][j][i][k].clear();
				infoDetec[1][j][i][k].clear();
			}
		}
    }
    // loop
    sniff->loop(readDump,(u_char*)this,fileInd,offset);
    // Refresh pictures : 
    (*menuList)[2] = tmp;
    if(loopMode == 'i'){
	yRange[6] = 0;yRange[7] = 0;
	for(int i=0; i<mainStat->dimx(); i++){
		(*mainStat)(i,5) = (double)(*mainStat)(i,5)/(double)(*mainStat)(i,6);
		if ((*mainStat)(i,5) > yRange[6]) yRange[6] = (*mainStat)(i,5);
		for(int j=7; j<15; j++)
			if ((*mainStat)(i,j) > yRange[7]) yRange[7] = (*mainStat)(i,j);
	}
	mainImg[6].draw_graph(mainStat->get_lines(5,5),black,1,yRange[6],0);	
	for(int i=0; i<mainStat->dimx(); i++) (*mainStat)(i,4) /= timeCoef*1024./8.;		// kbits/sec
	yRange[5] /= timeCoef*1024./8.;
	mainImg[5].draw_graph(mainStat->get_lines(4,4),black,1,yRange[5],0);
	
	mainImg[4].draw_graph(mainStat->get_lines(0,0),red,1,yRange[4],0);
	mainImg[4].draw_graph(mainStat->get_lines(1,1),blue,1,yRange[4],0);
	mainImg[4].draw_graph(mainStat->get_lines(2,2),green,1,yRange[4],0);
	mainImg[4].draw_graph(mainStat->get_lines(3,3),black,1,yRange[4],0);

	//mainImg[7].draw_graph(mainStat->get_lines(7,7),red,1,yRange[7],0);
	//mainImg[7].draw_graph(mainStat->get_lines(8,8),blue,1,yRange[7],0);
	mainImg[7].draw_graph(mainStat->get_lines(9,9),green,1,yRange[7],0);
	mainImg[7].draw_graph(mainStat->get_lines(10,10),orange,1,yRange[7],0);
	mainImg[7].draw_graph(mainStat->get_lines(11,11),purple,1,yRange[7],0);
	mainImg[7].draw_graph(mainStat->get_lines(12,12),last,1,yRange[7],0);
	mainImg[7].draw_graph(mainStat->get_lines(13,13),grey,1,yRange[7],0);
	mainImg[7].draw_graph(mainStat->get_lines(14,14),black,1,yRange[7],0);

	/*	if(tcpHdr->tcpFlags & TH_SYN)	(*gui->mainStat)(x,7) ++;
		if(tcpHdr->tcpFlags & TH_ACK)	(*gui->mainStat)(x,8) ++;
		if(tcpHdr->tcpFlags & TH_FIN)	(*gui->mainStat)(x,9) ++;
		if(tcpHdr->tcpFlags & TH_RST)	(*gui->mainStat)(x,10) ++;
		if(tcpHdr->tcpFlags & TH_PUSH)	(*gui->mainStat)(x,11) ++;
		if(tcpHdr->tcpFlags & TH_URG)	(*gui->mainStat)(x,12) ++;
		if(tcpHdr->tcpFlags & TH_ECE)	(*gui->mainStat)(x,13) ++;
		if(tcpHdr->tcpFlags & TH_CWR)	(*gui->mainStat)(x,14) ++;*/

	displayRefresh(-1);
    }
    else if(loopMode == 'a'){
	pthread_rwlock_t rwAnVec;
	pthread_rwlock_init(&rwAnVec, NULL);
	//!Detector::rwAnVec = rwAnVec;

	//!detector1.detect(detecImg[0],infoDetec[0],&pktList,0);
	//mainDisp[0].display(detector.imgIdentified);
	//!detector2.detect(detecImg[1],infoDetec[1],&pktList,10);
	//mainDisp[1].display(detector.imgIdentified);
	
	//!detector1.wait();
	//!detector2.wait();

	//Remove anomalies with only one event
	static int clean=0;
	if(clean<cleanFreq) clean++;
	else{
		//!detector1.anomaliesClean();
		clean=0;
	}

	//cout << detector1.leaderSrc.size() << "anomalies (src) found" << endl;
	//cout << detector1.leaderDst.size() << "anomalies (dst) found" << endl;
	pktList.clear();	// release some memory ??
   }
}

void GUI::wait(){
    std::ostringstream 	strTmp;

    refreshMenu = false;
    CImgDisplay::wait(*menuDisp,mainDisp[0]);
    yCoefI = yCoef[yMode[0]-'a'];
    timeCoefI = timeCoef;
    if (menuDisp->is_event && menuDisp->mouse_y >= 0 && menuDisp->mouse_y < menuY+32){
	eventDisplay 	= 1;
	if (menuDisp->mouse_y < 32){
		strTmp << "btn" << menuDisp->mouse_x/32 + 1;
		event  = strTmp.str();
		if(menuDisp->button)
			eventButton	= menuDisp->button;
		else 	eventButton	= -1;
	}
	else{
		event   = "map";
		if(menuDisp->button){
			eventButton= menuDisp->button;
			mouseYR[yMode[0]-'a']= (menuDisp->mouse_y-32)-((yCoef[yMode[0]-'a']/yCoef0[yMode[0]-'a'])*menuY)/2;
			mouseY[yMode[0]-'a'] = ((menuY+32)-menuDisp->mouse_y)-((yCoef[yMode[0]-'a']/yCoef0[yMode[0]-'a'])*menuY)/2;
			mouseX = menuDisp->mouse_x-((timeCoef / timeCoef0)*menuX)/2;
		}
		else 	eventButton	= -1;
	}
    }
    else if(mainDisp[0].wheel){
		eventDisplay 	= 2;
		eventButton 	= mainDisp[0].wheel;
		event   	= "whl";
		mainDisp[0].wheel = 0;
    }
    else if(mainDisp[0].button && mainDisp[0].mouse_y >= 0 ){
		eventDisplay 	= 2;
		eventButton 	= mainDisp[0].button;
		event   	= "map";
		mainDisp[0].button= 0;
    }
    else{
	eventDisplay 	= -1;
	eventButton	= -1;
	event   = "";
    }
}

void GUI::saveImg(){
    saveImg(0);
    saveImg(1);
}

void GUI::saveImg(int disp){
    char date[40];
    time_t now = time(NULL);
    struct tm * timeinfo;
    fstream fin;
    int count = 0;
    string info("Picture saved in : ");

    timeinfo = localtime (&now);
    strftime(date,40,"screenshot/%y_%m_%d_%X.png",timeinfo);
    fin.open(date,ios::in);
    while( fin.is_open() ){	// File already exist
	fin.close();
	strftime(date,40,"screenshot/%y_%m_%d_%X_%%d.png",timeinfo);
	sprintf(date,date,count++);
	fin.open(date,ios::in);
    }
    if(	glassX != -1 && disp == 1){
	// save the graph corresponding to the glass
	CImg<unsigned char> tmp;
	mainDisp[1].snapshot(tmp);
	tmp.save_png(date);
    }
    else{
    	if(saveRulers) mainList[disp].get_append('y','n').save_png(date);
    	else mainImg[yMode[disp]-'a'].save_png(date);
    }

    infoDisplay(info.append(date).c_str(),11);
    if(!silent) cout << info << endl;
}

void GUI::rtz(){
    timeCoef = timeCoef0;
    yCoef[yMode[0]-'a']  = yCoef0[yMode[0]-'a'];
    mouseX = 0;
    mouseY[yMode[0]-'a'] = 0;
    mouseYR[yMode[0]-'a']= 0;
}

void GUI::focusMouse(){
    int x = mainDisp[0].mouse_x;
    int y = mainDisp[0].mouse_y;
    mouseX += (((x - 15) - mainX/2)/((double)mainX/(double)menuX))*(timeCoef / timeCoef0);
    mouseY[yMode[0]-'a'] += (((mainY - y) - mainY/2)/((double)mainY/(double)menuY))*(yCoef[yMode[0]-'a']/yCoef0[yMode[0]-'a']);
    mouseYR[yMode[0]-'a']+= ((y - mainY/2)/((double)mainY/(double)menuY))*(yCoef[yMode[0]-'a']/yCoef0[yMode[0]-'a']);
}

void GUI::focusCenter(){
    mouseX += (((timeCoefI / timeCoef0)-((timeCoef / timeCoef0)))*(menuX))/2;
    mouseY[yMode[0]-'a'] += ((yCoefI/yCoef0[yMode[0]-'a'] - yCoef[yMode[0]-'a']/yCoef0[yMode[0]-'a'] )/2.)*(double)menuY;
    mouseYR[yMode[0]-'a']+= ((yCoefI/yCoef0[yMode[0]-'a'] - yCoef[yMode[0]-'a']/yCoef0[yMode[0]-'a'] )/2.)*(double)menuY;
}

void GUI::stepX(double step){
    mouseX += step;
}

void GUI::stepY(double step){
    mouseY[yMode[0]-'a'] -= step;
    mouseYR[yMode[0]-'a']+= step;
}

void GUI::zoomInX(double step, bool center){ 
    if (step <= 0) return;
    timeCoef /= step;
    if(center)	focusCenter();
}

void GUI::zoomOutX(double step, bool center){
    if (step <= 0) return;
    timeCoef *= step;
    if(center)	focusCenter();
}

void GUI::zoomInY(double step, bool center){
    if (step <= 0 || yMode[0] > 'd') return;
    yCoef[yMode[0]-'a'] /= step;
    if(center)	focusCenter();
}

void GUI::zoomOutY(double step, bool center){
    if (step <= 0  || yMode[0] > 'd') return;
    yCoef[yMode[0]-'a'] *= step;
    if(center)	focusCenter();
}

void GUI::zoomWhl(){
    timeCoefI = timeCoef;
    yCoefI = yCoef[yMode[0]-'a'];
    if(eventButton>0){
	yCoef[yMode[0]-'a'] /= 1.1;
	timeCoef /= 1.1;
    }
    else{
	yCoef[yMode[0]-'a'] *= 1.1;
	timeCoef *= 1.1;
    }
    focusCenter();
}

void GUI::nextYMode(int disp){
    yMode[disp] = yMode[disp]+1 > 'a'+NB_YMODE-1? 'a' : yMode[disp]+1;
}

void GUI::prevYMode(int disp){
    yMode[disp] = yMode[disp]-1 < 'a'? 'a'+NB_YMODE-1 : yMode[disp]-1;
}

/**
* Add data in 
*
*
*/
void GUI::graphIn(int x, int y){
	
	unsigned int srcPort = 0;
	unsigned int dstPort = 0;
	pair<info_type::const_iterator, info_type::const_iterator> p = info[y].equal_range((const int)x);

	for(info_type::const_iterator i= p.first; i != p.second; ++i){
		switch (i->second.proto) {
			case IPPROTO_TCP:
				srcPort = ntohs(i->second.hdr.tcp->tcpSrcPort);
				dstPort = ntohs(i->second.hdr.tcp->tcpDestPort);
				break;
			case IPPROTO_UDP: 
				srcPort = ntohs(i->second.hdr.udp->udpSrcPort);
				dstPort = ntohs(i->second.hdr.udp->udpDestPort);
				break;
		}
		// For TCP and UDP we record ip and port
		if(i->second.proto == IPPROTO_TCP || i->second.proto == IPPROTO_UDP){
			// Ip src  and port src
			if(!(graphSrcA[i->second.srcAddr]).count(srcPort))
				(graphSrcA[i->second.srcAddr])[srcPort] = 0;
			else if((graphSrcA[i->second.srcAddr]).count(srcPort)!=255)
				(graphSrcA[i->second.srcAddr])[srcPort]++;
			
			// Ip dst and port dst
			if(!(graphDstA[i->second.destAddr]).count(dstPort))
				(graphDstA[i->second.destAddr])[dstPort] = 0;
			else if((graphDstA[i->second.destAddr]).count(dstPort)!=255)
				(graphDstA[i->second.destAddr])[dstPort]++;
			
			// port src and port dst
			if(!(graphSrcP[srcPort]).count(dstPort))
				(graphSrcP[srcPort])[dstPort] = 0;
			else if((graphSrcP[srcPort]).count(dstPort)!=255)
				(graphSrcP[srcPort])[dstPort]++;

			//  port dst and port src
			if(!(graphDstP[dstPort]).count(srcPort))
				(graphDstP[dstPort])[srcPort] = 0;
			else if((graphDstP[dstPort]).count(srcPort)!=255)
				(graphDstP[dstPort])[srcPort]++;
		}
		// For other protocols we record only addresses
		else{
			// ip src and ip dst
			if(!(graphOther[i->second.srcAddr]).count(i->second.destAddr))
				(graphOther[i->second.srcAddr])[i->second.destAddr] = 0;
			else if((graphOther[i->second.srcAddr]).count(i->second.destAddr)!=255)
				(graphOther[i->second.srcAddr])[i->second.destAddr]++;
		}
		
	}
}

/**
* Draw the graph recorded in memory on the given display
*
*
*/
void GUI::graphDraw(int disp){
	CImg<unsigned char> graph(mainDisp[disp].dimx(),mainDisp[disp].dimy(),1,3,255);
	
	// Plot source address
	int index = 0;
	int stepHeight = graphSrcA.size()-1?graphSrcA.size()-1:1;
	unsigned int radius;
	for(map<InetAddr,map<unsigned int, char> >::const_iterator i= graphSrcA.begin(); i != graphSrcA.end(); i++){
		//draw edges and compute the radius of the vertice:
		radius = 0;
		int x0 = 20;
		int y0 = 20+(index*(mainDisp[disp].dimy()-30))/stepHeight;
		int stepHeightPort = graphSrcP.size()-1?graphSrcP.size()-1:1;
		for(map<unsigned int, char>::const_iterator j= i->second.begin(); j != i->second.end(); j++){
			radius += j->second+1;
			
			int x1 = mainDisp[disp].dimx()/3;
			int y1 = 20+(std::distance(graphSrcP.begin(),graphSrcP.find(j->first))*(mainDisp[disp].dimy()-30))/stepHeightPort;
			graph.draw_line(x0,y0,x1,y1,grey);
		}

		// draw the vertice:
		graph.draw_circle(x0,y0,radius,black);	//outline of the circle
		graph.draw_circle(x0,y0,radius-1,grey);	//inside
		ostringstream label;
		label << i->first;
		graph.draw_text(label.str().c_str(),x0-10,y0,black);

		index++;
	}


	// Plot source port
	index = 0;
	stepHeight = graphSrcP.size()-1?graphSrcP.size()-1:1;
	for(map<unsigned int,map<unsigned int, char> >::const_iterator i= graphSrcP.begin(); i != graphSrcP.end(); i++){
		//draw edges and compute the radius of the vertice:
		radius = 0;
		int x0 = mainDisp[disp].dimx()/3;
		int y0 = 20+(index*(mainDisp[disp].dimy()-30))/stepHeight;
		int stepHeightPort = graphDstP.size()-1?graphDstP.size()-1:1;
		for(map<unsigned int, char>::const_iterator j= i->second.begin(); j != i->second.end(); j++){
			radius += j->second+1;
			
			int x1 = 2*mainDisp[disp].dimx()/3;
			int y1 = 20+(std::distance(graphDstP.begin(),graphDstP.find(j->first))*(mainDisp[disp].dimy()-30))/stepHeightPort;
			graph.draw_line(x0,y0,x1,y1,grey);
		}

		// draw the vertice:
		graph.draw_circle(x0,y0,radius,black);
		graph.draw_circle(x0,y0,radius-1,grey);
		ostringstream label;
		label << i->first;
		graph.draw_text(label.str().c_str(),x0-10,y0,black);
		index++;
	}

	// Plot destination address
	index = 0;
	stepHeight = graphDstA.size()-1?graphDstA.size()-1:1;
	for(map<InetAddr,map<unsigned int, char> >::const_iterator i= graphDstA.begin(); i != graphDstA.end(); i++){
		//draw edges and compute the radius of the vertice:
		radius = 0;
		int x0 = mainDisp[disp].dimx()-20;
		int y0 = 20+(index*(mainDisp[disp].dimy()-30))/stepHeight;
		int stepHeightPort = graphDstP.size()-1?graphDstP.size()-1:1;
		for(map<unsigned int, char>::const_iterator j= i->second.begin(); j != i->second.end(); j++){
			radius += j->second+1;
			
			int x1 = 2*mainDisp[disp].dimx()/3;
			int y1 = 20+(std::distance(graphDstP.begin(),graphDstP.find(j->first))*(mainDisp[disp].dimy()-30))/stepHeightPort;
			graph.draw_line(x0,y0,x1,y1,grey);
		}

		// draw the vertice:
		graph.draw_circle(x0,y0,radius,black);
		graph.draw_circle(x0,y0,radius-1,grey);
		ostringstream label;
		label << i->first;
		graph.draw_text(label.str().c_str(),x0-60,y0,black);
		index++;
	}

	// Plot destination port
	index = 0;
	stepHeight = graphDstP.size()-1?graphDstP.size()-1:1;
	for(map<unsigned int,map<unsigned int, char> >::const_iterator i= graphDstP.begin(); i != graphDstP.end(); i++){
		//draw edges and compute the radius of the vertice:
		radius = 0;
		int x0 = 2*mainDisp[disp].dimx()/3;
		int y0 = 20+(index*(mainDisp[disp].dimy()-30))/stepHeight;
		for(map<unsigned int, char>::const_iterator j= i->second.begin(); j != i->second.end(); j++)
			radius += j->second+1;
		// draw the vertice:
		graph.draw_circle(x0,y0,radius,black);
		graph.draw_circle(x0,y0,radius-1,grey);
		ostringstream label;
		label << i->first;
		graph.draw_text(label.str().c_str(),x0-10,y0,black);
		index++;
	}


	// Print labels:
	graph.draw_text("Src IP",5,0,blue);
	graph.draw_text("Src Port",(mainDisp[disp].dimx()/3)-5,0,blue);
	graph.draw_text("Dst Port",(2*mainDisp[disp].dimx()/3)-5,0,blue);
	graph.draw_text("Dst IP",mainDisp[disp].dimx()-40,0,blue);

// Display the graph
	mainDisp[disp] << graph;
}

void GUI::graphClean(){
	graphSrcA.clear();
	graphDstA.clear();
	graphSrcP.clear();
	graphDstP.clear();
	graphOther.clear();
}

/**
 * Display the glass to get information about plotted pixels. If no parameters are given then the glass appears where the mouse has been clicked. Parameters X and Y are useful for a non interactive mode and correspond to the upper left corner of the glass.
 * 
 * @param x Horizontal coordinate of the upper left corner of the glass. (non interactive mode)
 * @param y Vertical coordinate of the upper left corner of the glass. (non interactive mode)
 */
void GUI::glassDisplay(int X, int Y){
    if(X==-1 && Y==-1){
	glassX = mainDisp[0].mouse_x - 15;
	glassY = mainDisp[0].mouse_y;
	mainList[0][0] *= 0.3f;
	int x;
	int y;
	for(int i=-glassSize; i<glassSize+1; i++){
		x = (glassX-10)+(i*20);
		for(int j=-glassSize; j<glassSize+1; j++){
			if(glassX+i>=mainImg[yMode[0]-'a'].dimx() || glassY+j>=mainImg[yMode[0]-'a'].dimy() || glassX+i<0 || glassY+j<0)	continue;
			y = (glassY-10)+(j*20);
			unsigned char rgb[] = {mainImg[yMode[0]-'a'](glassX+i,glassY+j,0,0),mainImg[yMode[0]-'a'](glassX+i,glassY+j,0,1),mainImg[yMode[0]-'a'](glassX+i,glassY+j,0,2)};
			mainList[0][0].draw_rectangle(x,y,x+20,y+20,rgb);
		}
	}
	mainDisp[0].display(mainList[0],'y','n');
	display('d',((glassX-glassSize)/((double)mainX/(double)menuX))*(timeCoef / timeCoef0));
	menuDisp->display(*menuList,'y','p');
    }
    else{	// non interactive mode
	glassX = X+glassSize/2;
	glassY = Y+glassSize/2;
	display('d',(X/((double)mainX/(double)menuX))*(timeCoef / timeCoef0));
    }

}

void GUI::glassInfo(int x, int y){
    if(x==-1 && y==-1){
	x = ceil((double)(mainDisp[0].mouse_x - (glassX+10)) / 20.0);
	y = ceil((double)(mainDisp[0].mouse_y - (glassY+10)) / 20.0);
    }
    //Clic out of the glass
    if(abs(x) > glassSize || abs(y) > glassSize){
	mainList[0][0] = mainList2[0].get_append();
	mainDisp[0].display(mainList[0],'y','n');
	glassX = -1;
	glassY = -1;
	displayRefresh(1);
    }
    // Clic in the glass
    else{
	ostringstream 	strInfo;
	info_type::const_iterator i;
	int		counter = 0;
	if(yMode[0] > 'd')	y = mainImg[yMode[0]-'a'].dimy()-1;
	else 			y = glassY+y;
	int		nbPkts = info[y].count((const int)glassX+x);

	strInfo << "Nb. packets :" << nbPkts << endl ;
	pair<info_type::const_iterator, info_type::const_iterator> p = info[y].equal_range((const int)glassX+x);
	for (i = p.first; i != p.second; ++i){
		switch (i->second.proto) {
			case IPPROTO_TCP:
				strInfo << i->second.srcAddr << ':' << ntohs(i->second.hdr.tcp->tcpSrcPort) << " > ";
				strInfo << i->second.destAddr << ':' << ntohs(i->second.hdr.tcp->tcpDestPort) << ' ';
				strInfo << "tcp " << printTcpFlag(i->second.hdr.tcp->tcpFlags) << ", ";
				strInfo << i->second.hdr.tcp->tcpSeq << ", ";
				strInfo << i->second.hdr.tcp->tcpAck << ' ';
				break;
			case IPPROTO_UDP: 
				strInfo << i->second.srcAddr << ':' << ntohs(i->second.hdr.udp->udpSrcPort) << " > ";
				strInfo << i->second.destAddr << ':' << ntohs(i->second.hdr.udp->udpDestPort) << ' ';
				strInfo << "udp " << ' ';
				break;
			case IPPROTO_ICMP:
				strInfo << i->second.srcAddr << " > ";
				strInfo << i->second.destAddr << ' ';
				strInfo << "icmp " << i->second.hdr.icmp->icmpInfo << ' ';
				break;
			default : 
				strInfo << i->second.srcAddr << " > ";
				strInfo << i->second.destAddr << ' ';
				strInfo << (int)i->second.proto << ' ';
				break;
		}
		strInfo << '(' << i->second.len << ", " << i->second.ts.tv_sec << '.' << i->second.ts.tv_usec  << ')' << "\n";
		counter++;
		if(counter==40){
			if(silent) infoDisplay(strInfo.str().append("... (For more details please disable the silent option.)").c_str(),462);
			else infoDisplay(strInfo.str().append("... (Please see the standard output for more details.)").c_str(),462);
		}
	}
	if(!silent)cout << strInfo.str() << endl;
	if(counter<40){
		infoDisplay(strInfo.str().c_str(),11*(counter+2));
	}
	menuDisp->display(*menuList,'y','p');

	// Plot the corresponding graph
	if(mainDisp[0].key != 65507)
		graphClean();
	
	graphIn(glassX+x,y);
	graphDraw(1);

    }
}

/*!
    \fn GUI::infoDisplay()
 */
void GUI::infoDisplay(const char* info, int size, bool empty, int x, int y)
{
    if (!menuDisp) return;
    if (empty){
	(*menuList)[2] = infoImg->get_resize(menuX,size).fill(0).draw_text(info,x,y,white);
	menuDisp->resize(menuX,menuY+32+size,false);
    }
    else{
	(*menuList)[2] = (*menuList)[2].draw_text(info,x,y,red);
    }
    menuDisp->display(*menuList,'y','p');
}

/**
 * 
 * @return 
 */
bool GUI::isClosed(){
    return menuDisp->is_closed || mainDisp[0].is_closed;
}

void GUI::help(){
    CImg<unsigned char> help("img/help2.png");
    help.display("Help");
}

/*!
    \fn GUI::rulersDisplay
 */
void GUI::rulersDisplay(char mode,int disp){
    int unit;
    double	actualTimeRange = (timeCoef/timeCoef0)*(double)timeRange;
    if (actualTimeRange > 129600) {
	unit = 7200;
    } else if (actualTimeRange > 43200) {
	unit = 3600;
    } else if (actualTimeRange > 21600) {
	unit = 1800;
    } else if (actualTimeRange > 10800) {
	unit = 600;
    } else if (actualTimeRange > 2700) {
	unit = 300;
    } else if (actualTimeRange > 600) {
	unit = 60;
    } else if (actualTimeRange > 120) {
	unit = 30;
    } else if (actualTimeRange > 30) {
	unit = 10;
    } else unit = 5;

    double	offset = unit-(startTick%unit);
    float 	pixPerSec = (float)mainX/actualTimeRange;
    time_t	tmp = startTick+offset;
    char	time[9];

    // Ruler for the x axis
    rulerXImg->fill(0);

    time[8] = '\0';time[5] = '\0';
    for (time_t i=(unsigned int)offset*pixPerSec; i<mainX ; i+=unit*pixPerSec) {
	if(unit<60)
		strncpy(time,&(ctime(&tmp)[11]),8);
	else	strncpy(time,&(ctime(&tmp)[11]),5);
	tmp += unit;
	rulerXImg->draw_line(i,0,i,5,white);
	rulerXImg->draw_text(time,i-11,5,white);
    }
    mainList[disp][1] = *rulerXImg;

    // Ruler for the y axis
    rulerYImg->fill(0);	
    if(mode == 'a' || mode == 'c'){		//Address
	char ip[15];
	int bitSignf = mode == 'a' ? cidrDest : cidrSrc;
	
	offset = (0xffffffff >> bitSignf) * (mouseY[yMode[disp]-'a']/(double)menuY + yCoef[mode-'a']/yCoef0[mode-'a']);
	if(offset>=0){
		printIp(offset,bitSignf,ip);
		rulerYImg->draw_text(ip,mainY-strlen(ip)*6,0,white);
		rulerYImg->draw_line(mainY-1,15,mainY-1,10,white);
	}
	
	offset = (0xffffffff >> bitSignf) * mouseY[yMode[disp]-'a']/(double)menuY;
	if(offset>=0){
		printIp(offset,bitSignf,ip);
		rulerYImg->draw_text(ip,5,0,white);
		rulerYImg->draw_line(0,15,0,10,white);
	}
    }
    else{
	char port[6];

	for(double ind=0; ind<=10; ind++){
		offset = yRange[mode-'a'] * (mouseY[mode-'a']/(double)menuY + ind/10.0 * yCoef[mode-'a']/yCoef0[mode-'a']);
		if (offset < 0) continue;
		sprintf( port,"%d", (int)offset);

		if(ind==0 || ind==10){
			rulerYImg->draw_line((ind/10.0)*mainY-0.1*(double)ind,15,(ind/10.0)*mainY-0.1*(double)ind,10,white);
			rulerYImg->draw_text(port,(ind/10.0)*mainY-strlen(port)*(0.6*(double)ind),0,white);
		}
		else{
			rulerYImg->draw_line((ind/10.0)*mainY,15,(ind/10.0)*mainY,10,white);
			rulerYImg->draw_text(port,(ind/10.0)*mainY-strlen(port)*4,0,white);
		}
	}
    }
    mainList2[disp][0] = rulerYImg->get_rotate(270);
    mainList[disp][0] = mainList2[disp].get_append();
}
