//
// C++ Implementation: mulot
//
// Description: 
//
//
//     Copyright (C) 2008  Romain Fontugne <romain.fontugne@gmail.com>
//
//     This program is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     This program is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
#include <stdlib.h>
#include <stdarg.h>

#include <zlib.h>
#include "sniffer.h"
#include "mulot.h"
#include "gui.h"


int strSwitch(int num,string str, ...){
    va_list ap;
    va_start(ap, &str);
    for(int i = 1; i<=num; i++)
	if(!str.compare(va_arg(ap,char*))) return i;
    return 0;
} 

void saveScreenshot(GUI* gui,int ngMode){
    if(ngMode != 7){
	gui->display();
	gui->saveImg();
    }
    else{
	for(int j=0; j<6; j++){
	    gui->display();
	    gui->saveImg();
	    gui->nextYMode(0);
	}
    }
}

unsigned long fileSize(char *filename){
    FILE *pFile = fopen(filename, "rb");
    fseek (pFile, 0, SEEK_END);
    unsigned long size = ftell(pFile);
    fclose (pFile);
    return size;
}

char* decompressFile(char *infilename){
    gzFile infile = gzopen(infilename, "rb");
    int outfile;
    char* fName	  = new char[13];
    #ifdef __MINGW32__
    static char count = 'a';
    strcpy(fName,"tmp\\VPX");
    fName[6] = count++;
    outfile = open(fName,O_CREAT | O_WRONLY,00777);
    #else
    strcpy(fName,"tmp/VPXXXXXX");    
    outfile = mkstemp(fName);
    #endif

    if (!infile || outfile==-1) return NULL;

    char buffer[128];
    int num_read = 0;
    while ((num_read = gzread(infile, buffer, sizeof(buffer))) > 0) {
	write(outfile, buffer, num_read);
    }
    close(outfile);
    gzclose(infile);
    return fName;
}

//
//	main routine
//		argument analysis, setup and main loop
//
int
main(int argc, char *argv[])
{
    char	opt;		// option character
    double 	step = 1;
    Sniffer 	sniff;
    GUI		gui;
    int		ngMode = 0;
    bool	concat = false;
    bool	download = false;

    //!Detector::htThreshold = 1.0;
    //!Detector::htWeight    = 1.0;

    while ((opt = getopt(argc, argv, "A:a:f:t:X:Y:x:y:g:b:B:sdcmnS:rT:W:h")) != -1)
	switch (opt) {
	    case 'A':		// output annotation file
		//!gui.detector1.annotationFile = (char*)malloc(strlen(optarg)+1);
		gui.nbAnnotation = 1;
		//!strncpy(gui.detector1.annotationFile,optarg,strlen(optarg)+1);
		break;
	    case 'a':		// input annotation file
//		gui.annotation = admd::annotation_t::read(optarg);
		gui.nbAnnotation = 1;
		break;
	    case 'f':		// filter code
		sniff.code = optarg;
		break;
	    case 't':		// time range
		gui.timeRange = atoi(optarg);
		if(gui.timeRange <= 0){
			cout << "Invalid parameter (time_range). It should be positif." << endl;
			exit(0);
		}
		break;
	    case 'X':		// width
		gui.mainX = atoi(optarg);
		if(gui.mainX <= 0){
			cout << "Invalid parameter (resX). It should be positif." << endl;
			exit(0);
		}
		break;
	    case 'Y':		// height
		gui.mainY = atoi(optarg);
		if(gui.mainY <= 0){
			cout << "Invalid parameter (resY). It should be positif." << endl;
			exit(0);
		}
		break;
	    case 'x':		// width
		gui.menuX = atoi(optarg);
		if(gui.menuX <= 0){
			cout << "Invalid parameter (resX). It should be positif." << endl;
			exit(0);
		}
		break;
	    case 'y':		// height
		gui.menuY = atoi(optarg);
		if(gui.menuY <= 0){
			cout << "Invalid parameter (resY). It should be positif." << endl;
			exit(0);
		}
		break;
	    case 'g':		// glass Size
		gui.glassSize = atoi(optarg);
		if(gui.glassSize <= 0){
			cout << "Invalid parameter (radius). It should be >=1." << endl;
			exit(0);
		}
		break;
	    case 'b':		// significant bit
		gui.cidrDest = atoi(optarg);
		if(gui.cidrDest < 0 || gui.cidrDest > 32){
			cout << "Invalid parameter (cidr). It should be between 0 and 32." << endl;
			exit(0);
		}
		break;
	    case 'B':		// significant bit
		gui.cidrSrc = atoi(optarg);
		if(gui.cidrSrc < 0 || gui.cidrSrc > 32){
			cout << "Invalid parameter (cidr). It should be between 0 and 32." << endl;
			exit(0);
		}
		break;
	    case 's':		// silent mode
		gui.silent = true;
		break;
	    case 'd':		// copy files in temporary directory
		download = true;
		break;
	    case 'm':		// monocolor
			gui.colorMode = 'm';
		break;
	    case 'n':		// Detection without gui
			ngMode = -1;
		break;
	    case 'S':		// non graphical mode
		ngMode = atoi(optarg);
		if(ngMode < 1 || ngMode > 7){
			cout << "Invalid parameter (mode). It should between 1 and 7." << endl;
			exit(0);
		}
		break;
	    case 'c':		// concat files
		concat = true;
		break;
	    case 'r':		// no rulers for screenshot
		gui.saveRulers = false;
		break;
	    case 'T':		// relative threshold for the hough transform
		//!Detector::htThreshold  = atof(optarg);
		break;
	    case 'W':		// Weight for the hough transform
		//!Detector::htWeight  = atof(optarg);
		break;
	    case 'h':
	    default:
		std::cout << "Usage : mulot [-A annotationOut] [-a annotationIn] [-f filter_code] [-t time_range]"
			<< "[-X resX] [-Y resY] [-x resX] [-y resY] "
			<< "[-g radius] [-b cidr] [-B cidr] [-s] [-d] [-r] [-n] [-S mode [-c]]"
			<< " dump_file... \n" << std::endl;
		std::cout << "Options : \n"
			<< "   -A \t annotationOut is the output file for the anomaly detection (admd format)"
			<< "   -a \t annotationIn is an admd format file providing anomaly description"
			<< "   -f \t filter_code is a tcpdump filter to consider wich packet will be displayed.\n"
			<< "   -t \t time_range is the time range of the menu display in seconds.\n"
			<< "\t (default value : 86400, means one day)\n"
			<< "   -X \t resX is the width of the main display.\n"
			<< "   -Y \t resY is the height of the main display.\n"
			<< "   -x \t resX is the width of the menu display.\n"
			<< "   -y \t resY is the height of the menu display.\n"
			<< "   -g \t radius is the size of the 'glass' in pixel.\n"
			<< "\t We call 'glass' the zoomed part of the picture showed to know data's information. (default value : 7)\n"
			<< "   -b \t cidr is the prefix length (number of shared initial bits) to use for destination IP addresses. (default value : 0)\n"
			<< "   -B \t cidr is the prefix length (number of shared initial bits) to use for source IP addresses. (default value : 0)\n"
			<< "   -s \t Enables silent mode.\n"
			<< "   -d \t Copy files to a temporary directory.\n"
			<< "   -r \t Remove rulers in screenshot.\n"
			<< "   -n \t Non-interactive mode. In order to identify traffic anomalies.\n"
			<< "   -S \t Non-interactive mode. In order to generate screenshot quickly.\n"
			<< "\t mode is the type of screenshot to generate :\n"
			<< "\t\t 1 : Destination IP address / Time.\n"
			<< "\t\t 2 : Destination port / Time.\n"
			<< "\t\t 3 : Source IP address / Time.\n"
			<< "\t\t 4 : Source port / Time.\n"
			<< "\t\t 5 : Number of packets / Time.\n"
			<< "\t\t 6 : Rate (bytes) / Time.\n"
			<< "\t\t 7 : All of them.\n"
			<< "   -c \t Concatenate files to make only one screenshot.\n"
			<< "\t By default one screenshot is taken for each file.\n"
			<< "   -T \t Relative threshold for the Hough Transform.\n"
			<< "   -W \t Weight of vote for the Hough Transform.\n"
			<< std::endl;
		exit(0);
	}

    int 	nbFiles = argc-optind;
    char*	files[nbFiles];
    for(int i=optind; i<argc; i++){			// Uncompress gzfiles
	string file(argv[i]);
	string::size_type s = file.find(".gz");
	if(s != string::npos || download){
		if(s != string::npos) 	cout << "uncompressing file : " << argv[i] << endl;
		else 			cout << "copying file : " << argv[i] << endl;
		if ((files[i- optind] = decompressFile(argv[i])) == NULL){
			if(s != string::npos) 	cout << "uncompress failed!" << endl;
			else 			cout << "copy failed!" << endl;
			exit(1);
		}
	}
	else files[i- optind] = argv[i];
    }
	ostringstream 	strCmd;
	for(int i=0; i<argc; i++)
		strCmd << argv[i] << " ";
	strCmd << endl;

	//!gui.detector1.parameters = strCmd.str();

    if(ngMode){	// Non-interactif mode.
	gui.sniff	= &sniff;
	gui.init(false);

	// Detection of traffic anomalies
	if(ngMode == -1){
	//	Display arguments passed to the application
		cout << "#" << strCmd.str();		
		time_t now = time(NULL);

		cout << endl << "#Detection starts on " << ctime (&now) << endl;

		sniff.nbFiles	= nbFiles;
		sniff.files	= files;

		unsigned int fileDuration = 900;// TODO change 900 by the total length of traces
		// TODO make the distinction of protocol in the image generation?
		// TODO split protocol for visualisation as well
// 		sniff.code = "tcp";
 		sniff.init();
 		for(unsigned int i=0; i<fileDuration/gui.timeRange; i++){gui.display('a'); gui.stepX(gui.menuX);}

// 		sniff.code = "udp";
// 		sniff.init();
// 		for(unsigned int i=0; i<fileDuration/gui.timeRange; i++){ gui.display('a'); gui.stepX(gui.menuX);}

// 		sniff.code = "!tcp and !udp";
//  		sniff.init();
//  		for(unsigned int i=0; i<fileDuration/gui.timeRange; i++){ gui.display('a'); gui.stepX(gui.menuX);}

		now = time(NULL);
		cout << endl << "#Detection finishs on " << ctime (&now) << endl;
		//!cout << "#Nb. anomalies in clusters: "<< gui.detector1.clustering() << endl;
		//!gui.detector1.printCluster(true);
		//gui.detector1.printHeuristic();
		//gui.detector1.anomalyPrint('o');
		//!gui.detector1.recCluster();

	}
	// Batch screenshots
	else{
		for(int i=1; i<ngMode && ngMode!=7; i++)
			gui.nextYMode(0);
		if(concat){
			sniff.nbFiles	= nbFiles;
			sniff.files	= files;
			sniff.init();
			saveScreenshot(&gui,ngMode);
		}
		else{
			for(int i=0; i<argc - optind; i++){
				sniff.nbFiles	= 1;
				sniff.files	= &files[optind+i];
				sniff.init();
				saveScreenshot(&gui,ngMode);
			}
		}
	}
    }
    else{
	sniff.nbFiles	= nbFiles;
	sniff.files	= files;
	sniff.init();
	gui.sniff	= &sniff;
	gui.init();

	while (!gui.isClosed()) {
		gui.wait();
		if (gui.eventDisplay == 1) {
			if (gui.eventButton == 1) step = 1; else if(gui.eventButton != -1) step = 10; else step = -1;
			int ind = strSwitch(13,gui.event, "btn1", "btn2", "btn3", "btn4", "btn5", "btn6", "btn7", "btn8", "btn9", "btn10", "btn11", "btn12", "btn13", 0);
			switch (ind){
				case 1:			// Zoom in regarding x axis
					gui.zoomInX(step*1.1);
					gui.infoDisplay("Zoom in on the x axis",11);
					break;
				case 2:			// Zoom out regarding x axis
					gui.zoomOutX(step*1.1);
					gui.infoDisplay("Zoom out on the x axis",11);
					break;
				case 3:			// Zoom in regarding y axis
					gui.zoomInY(step*1.1);
					gui.infoDisplay("Zoom in on the y axis",11);
					break;
				case 4:			// Zoom out regarding y axis
					gui.zoomOutY(step*1.1);
					gui.infoDisplay("Zoom out on the y axis",11);
					break;
				case 5:			// Navigation : left
					if (step != -1) gui.stepX(-step);
					gui.infoDisplay("Go left",11);
					break;
				case 6:			// Navigation : right
					if (step != -1) gui.stepX(step);
					gui.infoDisplay("Go right",11);
					break;
				case 7:			// Navigation : up
					if (step != -1) gui.stepY(-step);
					gui.infoDisplay("Go up",11);
					break;
				case 8:			// Navigation : up
					if (step != -1) gui.stepY(step);
					gui.infoDisplay("Go down",11);
					break;
				case 9:			// Home
					if(gui.eventButton != -1) gui.rtz();
					if(ind==9) gui.infoDisplay("Reset",11);
					break;
				case 10:		// Change filter mode
					if(gui.eventButton == 1){ gui.nextYMode(0); gui.refreshMenu = true;}
					else if(gui.eventButton != -1){ gui.prevYMode(0); gui.refreshMenu = true;}
					else gui.infoDisplay("Change filter mode", 11);
					break;
				case 11:		// Change Mode
					if(gui.eventButton == 1){ gui.nextYMode(1);gui.displayRefresh(1);}
					else if(gui.eventButton != -1){ gui.prevYMode(1);gui.displayRefresh(1);}
					else gui.infoDisplay("Change aux. mode", 11);
					break;
				case 12:		// Screenshot
					gui.infoDisplay("Take a screenshot",11);
					if(gui.eventButton != -1) gui.saveImg();
					break;
				case 13:
					gui.infoDisplay("Help",11);
					if(gui.eventButton != -1){
// 						cout << "time scale = 1 min" << endl;
// 						for(int i=0; i<15; i++){ gui.display('a'); gui.stepX(gui.menuX);}
// 						// Return to the begining of the file
// 						gui.rtz();
// 						gui.zoomOutX(5.0,false);	// time scale 5 times bigger
						//!cout << "time scale = " << gui.timeRange <<" sec" << endl;
						//!for(unsigned int i=0; i<900/gui.timeRange; i++){gui.display(); gui.display('a'); gui.stepX(gui.menuX);}
						// Return to the begining of the file
// 						gui.rtz();
// 						gui.zoomOutX(15.0,false);	// time scale 15 times bigger
// 						cout << "time scale = 15 min" << endl;
// 						gui.display('a');
						
						//!cout << "Nb. anomalies in clusters :"<< gui.detector1.clustering() << endl; 
						//!gui.detector1.printCluster(true); 
						//gui.detector1.printAllAnomaly();
						//!gui.detector1.recCluster();
						gui.help();
					}
			}
			if (ind<11 && gui.eventButton != -1) gui.display();
		}
		else if(gui.eventDisplay == 2){
			if(gui.glassX >= 0){
				gui.glassInfo();
			}
			// Zoom using the wheel
			else if(!gui.event.compare("whl")){
				gui.zoomWhl();
				gui.display();
			}
			else if (gui.eventButton == 1){
				// Focus on the part selected
				gui.focusMouse();
				gui.display();
			}
			else{
				// display glass
				gui.glassDisplay();
			}
		}
	}
    }

    return 0;
}
