//
//	common.cc
//		common routine
//		display address
//
//		$Id$
//

#include <iostream>
#include <iomanip>

#include "protohdr.h"

using namespace std;

EtherAddr	ETHER_BROADCAST(0xff, 0xff, 0xff, 0xff, 0xff, 0xff);
InetAddr	IP_MULTICAST_MIN(224, 0, 0, 0);
InetAddr	IP_MULTICAST_MAX(239, 255, 255, 255);
InetAddr	IP_PRIVATE_A_MIN(10, 0, 0, 0);
InetAddr	IP_PRIVATE_A_MAX(10, 255, 255, 255);
InetAddr	IP_PRIVATE_B_MIN(172, 16, 0, 0);
InetAddr	IP_PRIVATE_B_MAX(172, 31, 255, 255);
InetAddr	IP_PRIVATE_C_MIN(192, 168, 0, 0);
InetAddr	IP_PRIVATE_C_MAX(192, 168, 255, 255);

string ICMP_TYPE_TBL[] = {
    "echoreply",  "unknown(1)",    "unknown(2)", "unreach_type",
    "squench",    "redirect_type", "unknown(6)", "unknown(7)",
    "echo",       "rt_adv",        "rt_sol",     "time_xcd",
    "parm_prob",  "ts",            "ts_reply",   "info_req",
    "info_reply", "mask_req",      "mask_reply"
};

string ICMP_UNREACH_TBL[] = {
    "unreach_net",  "unreach_host", "unreach_proto", "unreach_port",
    "need_frag",    "srcrt_fail",   "net_unknown",   "host_unknown",
    "src_isolated", "net_prohibit", "host_prohibit", "bad_tos_net",
    "bad_tos_host", "filtered",     "prec_violate" , "prec_cutoff"
};

string ICMP_REDIRECT_TBL[] = {
    "rdirct_net", "rdirct_host", "rdirct_tos_net", "rdirct_tos_host"
};

std::ostream&
operator<<(std::ostream& s, EtherAddr const &ea) {
    s << std::hex
    << int(ea.octets[0]) << ':' << int(ea.octets[1]) << ':'
    << int(ea.octets[2]) << ':' << int(ea.octets[3]) << ':'
    << int(ea.octets[4]) << ':' << int(ea.octets[5]) << std::dec;
    return s;
}

std::ostream&
operator<<(std::ostream& s, EtherAddr& ea) {
    s << std::hex
    << int(ea.octets[0]) << ':' << int(ea.octets[1]) << ':'
    << int(ea.octets[2]) << ':' << int(ea.octets[3]) << ':'
    << int(ea.octets[4]) << ':' << int(ea.octets[5]) << std::dec;
    return s;
}

std::ostream&
operator<<(std::ostream& s, const InetAddr& ia) {
    s << int(ia.octets[0]) << '.' << int(ia.octets[1]) << '.'
    << int(ia.octets[2]) << '.' << int(ia.octets[3]) << std::dec;
    return s;
}

std::ostream&
operator<<(std::ostream& s, InetAddr& ia) {
    s << int(ia.octets[0]) << '.' << int(ia.octets[1]) << '.'
    << int(ia.octets[2]) << '.' << int(ia.octets[3]) << std::dec;
    return s;
}

std::ostream&
operator<<(std::ostream& s, IcmpType it) {
    if (it.typeData > IcmpType::maxtype)
        return(s << "unknown");
    else if (it.typeData == IcmpType::unreach) {
        if (it.typeData > IcmpType::maxunreach)
            return(s << "unreach_unknown");
        else
            return (s << ICMP_UNREACH_TBL[int(it.codeData)]);
    } else
        return(s << ICMP_TYPE_TBL[int(it.typeData)] << "(" << int(it.codeData) << ")");
}

const char* printTcpFlag(u_char flag){
string res = "";
if(flag & TH_FIN)	res.append("F");
if(flag & TH_SYN)	res.append("S");
if(flag & TH_RST)	res.append("R");
if(flag & TH_PUSH)	res.append("P");
if(flag & TH_ACK)	res.append("A");
if(flag & TH_URG)	res.append("U");
if(flag & TH_ECE)	res.append("E");
if(flag & TH_CWR)	res.append("C");

return res.c_str();
}

//
//	comparator
//
bool
operator<(EtherAddr ea1, EtherAddr ea2) {
    return(u_int64_t(ea1)<u_int64_t(ea2));
}

bool
operator>(EtherAddr ea1, EtherAddr ea2) {
    return(u_int64_t(ea1)>u_int64_t(ea2));
}

bool
operator==(EtherAddr ea1, EtherAddr ea2) {
    return(u_int64_t(ea1)==u_int64_t(ea2));
}

bool
operator<(InetAddr ia1, InetAddr ia2) {
    return(u_int32_t(ia1)<u_int32_t(ia2));
}

bool
operator>(InetAddr ia1, InetAddr ia2) {
    return(u_int32_t(ia1)>u_int32_t(ia2));
}

bool
operator==(InetAddr ia1, InetAddr ia2) {
    return(u_int32_t(ia1)==u_int32_t(ia2));
}

bool
operator==(IcmpType it1, const u_char it2) {
    return(int(it1.typeData)==int(it2));
}

//
//	test operator
//
bool
isEtherBroadcast(EtherAddr& aEaddr) {
    return(u_int64_t(aEaddr) == u_int64_t(ETHER_BROADCAST));
}

bool
isIpMulticast(InetAddr& aInetaddr) {
    return((u_int32_t(aInetaddr) >= u_int32_t(IP_MULTICAST_MIN)) &&
           (u_int32_t(aInetaddr) <= u_int32_t(IP_MULTICAST_MAX)));
}

bool
isIpBetween(InetAddr& aInetaddr, InetAddr& aInetMin, InetAddr& aInetMax) {
    return((u_int32_t(aInetaddr) >= u_int32_t(aInetMin)) &&
           (u_int32_t(aInetaddr) <= u_int32_t(aInetMax)));
}

bool
isIpPrivate(InetAddr& aInetaddr) {
    return(isIpPrivateA(aInetaddr) ||
           isIpPrivateB(aInetaddr) ||isIpPrivateC(aInetaddr));
}

bool
isIpPrivateA(InetAddr& aInetaddr) {
    return(isIpBetween(aInetaddr, IP_PRIVATE_A_MIN, IP_PRIVATE_A_MAX));
}

bool
isIpPrivateB(InetAddr& aInetaddr) {
    return(isIpBetween(aInetaddr, IP_PRIVATE_B_MIN, IP_PRIVATE_B_MAX));
}

bool
isIpPrivateC(InetAddr& aInetaddr) {
    return(isIpBetween(aInetaddr, IP_PRIVATE_C_MIN, IP_PRIVATE_C_MAX));
}
