//
//	protohdr.h
//
//		define protocol header fields
//
//		$Id$
//
#ifndef	_PROTOHDR_H
#define	_PROTOHDR_H

#include	<iostream>
#include	<unistd.h>
#include 	<netinet/in_systm.h>
//
//	Network types (borrowed from netinet/in_systm.h)
//
// to make mingw happy :
#ifdef __MINGW32__
typedef	uint16_t	u_int16_t;
typedef	uint32_t	u_int32_t;
typedef	uint64_t	u_int64_t;
typedef unsigned char           u_char;
typedef unsigned int            u_int;
typedef unsigned short int      u_short;
#endif

typedef	u_int16_t	n_short;	// short as received from the net
typedef	u_int32_t	n_long;		// long as received from the net

typedef	u_int32_t	n_time;		// ms since 00:00 GMT, byte rev

//
//	Header parameters
//

#define	ETHER_ADDR_LEN		6
#define ETHER_TYPE_LEN		2
#define ETHER_CRC_LEN		4
#define	ETHER_MAX_LEN		1518

/**
 *	Ethernet Adress
 */
struct EtherAddr {
	u_char	octets[ETHER_ADDR_LEN];
    public:
	EtherAddr(int e0 = 0, int e1 = 0, int e2 = 0, int e3 = 0,
					int e4 = 0, int e5 = 0) {
	    octets[0]=e0; octets[1]=e1; octets[2]=e2;
	    octets[3]=e3; octets[4]=e4; octets[5]=e5;
	}
	operator u_int64_t () {
	    return (((((u_int64_t)octets[0]<<8|octets[1])<<8|octets[2])<<8
			|octets[3])<<8|octets[4])<<8|octets[5];
	}
};

/**
 *	Internet Address (IPv4)
 */
struct InetAddr {
	u_char		octets[4];
    public:
	InetAddr(int i0 = 0, int i1 = 0, int i2 = 0, int i3 = 0) {
	    octets[0]=i0; octets[1]=i1; octets[2]=i2; octets[3]=i3;
	}
	operator u_int32_t () {
	    return ((octets[0]<<8|octets[1])<<8|octets[2])<<8|octets[3];
	}
	inline u_char	get1stOct()	{ return(octets[0]); }
	inline u_char	get2ndOct()	{ return(octets[1]); }
	inline u_char	get3rdOct()	{ return(octets[2]); }
	inline u_char	get4thOct()	{ return(octets[3]); }
	inline u_char	getOct(int n)	{ if (n < 0 || n > 3) return -1; else return(octets[n]); }
};

/**
 *	Ethernet Packet Header and parameters
 */
struct EtherHeader {
	EtherAddr	etherDest;
	EtherAddr	etherSrc;
	union {
	    u_int16_t	etherType;
	    u_int16_t	etherLlcLength;
	};
	union {
	    u_int16_t	etherPayload;
	    struct LlcHeader {
		u_char		llcDsap;
		u_char		llcSsap;
		u_char		llcControl;
		u_char		llcOrginalCode[3];
		u_int16_t	llcEtherType;
	    };
	} etherBody;
};

#define	ETHERTYPE_IP		0x0800
#define	ETHERTYPE_ARP		0x0806
#define	ETHERTYPE_VLAN		0x8100
#define	ETHERTYPE_IPX		0x8137

#define ETHER_HDR_LEN	(ETHER_ADDR_LEN*2+ETHER_TYPE_LEN)
#define	ETHERMTU	(ETHER_MAX_LEN-ETHER_HDR_LEN-ETHER_CRC_LEN)

/**
 *	IP Packet Header and Parameters
 */
struct IpHeader {
#ifdef _IP_VHL
	u_char		ipVhl;		// version << 4 | header length >> 2
#else	// not _IP_VHL
#if BYTE_ORDER == LITTLE_ENDIAN
	u_int		ipHeaderLength:4,	// header length 
			ipVersion:4;		// version
#elif BYTE_ORDER == BIG_ENDIAN
	u_int		ipVersion:4,		// version
			ipHeaderLength:4;	// header length
#endif
#endif	// not _IP_VHL
	u_char		ipTos;		// type of service
	u_int16_t	ipLength;	// total length
	u_int16_t	ipId;		// identification
	u_int16_t	ipFragOffset;	// fragment offset field
#define IP_RF 0x8000			// reserved fragment flag
#define IP_DF 0x4000			// dont fragment flag
#define IP_MF 0x2000			// more fragments flag
#define IP_OFFMASK 0x1fff		// img for fragmenting bits
	u_char		ipTtl;		// time to live
	u_char		ipProto;	// protocol
	u_int16_t	ipSum;		// checksum
	InetAddr	ipSrc;		// IP source address
	InetAddr	ipDest;		// IP destination address
};

// #define		IPPROTO_IP	0
// #define		IPPROTO_ICMP	1
// #define		IPPROTO_IGMP	2
// #define		IPPROTO_TCP	6
// #define		IPPROTO_UDP	17
// #define		IPPROTO_ESP	50

/**
 *	ICMP Packet Header and Parameters
 */
struct IcmpType {
	IcmpType(u_char	type = 0, u_char code = 0)	{
			typeData = type; codeData = code;
	};
	bool		isInfoType();

	u_char		typeData;
	u_char		codeData;

	static const u_char echoreply	= 0;	/* echo reply */
	static const u_char unreach	= 3;	/* dest unreachable, codes: */
	    static const u_char unreach_net	= 0;	/* bad net */
	    static const u_char unreach_host	= 1;	/* bad host */
	    static const u_char unreach_proto	= 2;	/* bad protocol */
	    static const u_char unreach_port	= 3;	/* bad port */
	    static const u_char need_flag	= 4;	/* IP_DF caused drop */
	    static const u_char srcrt_fail	= 5;	/* src route failed */
	    static const u_char net_unknown 	= 6;	/* unknown net */
	    static const u_char host_unknown	= 7;	/* unknown host */
	    static const u_char src_isolated	= 8;	/* src host isolated */
	    static const u_char net_prohibit	= 9;	/* prohibited access */
	    static const u_char host_prohibit	= 10;	/* ditto */
	    static const u_char bad_tos_net	= 11;	/* bad tos for net */
	    static const u_char bad_tos_host	= 12;	/* bad tos for host */
	    static const u_char filtered 	= 13;	/* admin prohib */
	    static const u_char prec_violate 	= 14;	/* host prec vio. */
	    static const u_char prec_cutoff 	= 15;	/* prec cutoff */

	    static const u_char maxunreach 	= 15;	/* max unreach code */

	static const u_char sourcequench = 4;	/* packet lost, slow down */
	static const u_char redirect	= 5;	/* shorter route, codes: */
	static const u_char echo	= 8;	/* echo service */
	static const u_char routeradvert = 9;	/* router advertisement */
	static const u_char routersolicit = 10;	/* router solicitation */
	static const u_char timxceed	= 11;	/* time exceeded, code: */
	static const u_char paramprob	= 12;	/* ip header bad */
	static const u_char tstamp	= 13;	/* timestamp request */
	static const u_char tstampreply	= 14;	/* timestamp reply */
	static const u_char ireq	= 15;	/* information request */
	static const u_char ireqreply	= 16;	/* information reply */
	static const u_char imgreq	= 17;	/* address img request */
	static const u_char imgreply	= 18;	/* address img reply */

	static const u_char maxtype	= 18;	/* max icmp type */
};

struct IcmpHeader {
	IcmpType	icmpInfo;
	u_short		icmpCksum;
};



#define	ICMP_INFOTYPE(type) \
	((type) == ICMP_ECHOREPLY || (type) == ICMP_ECHO || \
	(type) == ICMP_ROUTERADVERT || (type) == ICMP_ROUTERSOLICIT || \
	(type) == ICMP_TSTAMP || (type) == ICMP_TSTAMPREPLY || \
	(type) == ICMP_IREQ || (type) == ICMP_IREQREPLY || \
	(type) == ICMP_MASKREQ || (type) == ICMP_MASKREPLY)

//
//	TCP header
//
struct TcpHeader {
	u_int16_t	tcpSrcPort;	// source port
	u_int16_t	tcpDestPort;	// destination port
	u_int32_t	tcpSeq;		// sequence number
	u_int32_t	tcpAck;		// acknowledgement number
#if BYTE_ORDER == LITTLE_ENDIAN
	u_int		tcpReserved1:4,	// (unused)
			tcpOffset:4;	// data offset
#elif BYTE_ORDER == BIG_ENDIAN
	u_int		tcpOffset:4,	// data offset
			tcpReserved1:4;	// (unused)
#endif
	u_char		tcpFlags;
#define TH_FIN		0x01
#define TH_SYN		0x02
#define TH_RST		0x04
#define TH_PUSH		0x08
#define TH_ACK		0x10
#define TH_URG		0x20
#define TH_ECE		0x40
#define TH_CWR		0x80
#define TH_FLAGS	(TH_FIN|TH_SYN|TH_RST|TH_ACK|TH_URG|TH_ECE|TH_CWR)
#define	TH_NORMAL	(TH_FIN|TH_SYN|TH_RST|TH_ACK|TH_URG)

	u_int16_t	tcpWindow;	// window
	u_int16_t	tcpChecksum;	// checksum
	u_int16_t	tcpUrgentPtr;	// urgent pointer
};

//
//	UDP header
//
struct UdpHeader {
	u_int16_t	udpSrcPort;	// source port
	u_int16_t	udpDestPort;	// destination port
	u_int16_t	udpLength;	// UDP length
	u_int16_t	udpChecksum;	// UDP checksum
};

const char* printTcpFlag(u_char flag);
//
//	stream out
//
std::ostream& operator<<(std::ostream& s, const EtherAddr& ea);
std::ostream& operator<<(std::ostream& s, EtherAddr& ea);
std::ostream& operator<<(std::ostream& s, const InetAddr& ia);
std::ostream& operator<<(std::ostream& s, InetAddr& ia);
std::ostream& operator<<(std::ostream& s, IcmpType it);

//
//	comparator
//
bool operator<(EtherAddr ia1, const EtherAddr ia2);
bool operator>(EtherAddr ia1, const EtherAddr ia2);
bool operator==(EtherAddr ia1, const EtherAddr ia2);
bool operator<(InetAddr ia1, const InetAddr ia2);
bool operator>(InetAddr ia1, const InetAddr ia2);
bool operator==(InetAddr ia1, const InetAddr ia2);
bool operator==(IcmpType it1, const u_char it2);

//
//	test routine
//
bool isEtherBroadcast(EtherAddr& aEaddr);
bool isIpMulticast(InetAddr& aInetaddr);
bool isIpBetween(InetAddr& aInetaddr, InetAddr& aInetMin, InetAddr& aInetMax);
bool isIpPrivate(InetAddr& aInetaddr);
bool isIpPrivateA(InetAddr& aInetaddr);
bool isIpPrivateB(InetAddr& aInetaddr);
bool isIpPrivateC(InetAddr& aInetaddr);

#endif	// _PROTOHDR_H
