//
// C++ Implementation: sniffer
//
// Description: Class used in mulot to read tcpdump files.
//
//
//     Copyright (C) 2008  Romain Fontugne <romain.fontugne@gmail.com>
//
//     This program is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     This program is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
#include "sniffer.h"
#include <cstring>

Sniffer::Sniffer()
{
    code = NULL;
}

Sniffer::~Sniffer()
{
#ifdef __MINGW32__
    for(int i=0; i<nbFiles; i++){
        if(strncmp(files[i],"tmp\\VP",6)==0) unlink(files[i]);
    }
#endif
}
struct fileTs{
	pcap_t*	pDesc;
	int	ts;
    };

bool inf(const fileTs& a,const fileTs& b){
	return a.ts < b.ts;
}

int Sniffer::init(){
#ifndef __MINGW32__
    typedef vector<fileTs> ft_type;
    ft_type  tmp;
    char errBuffer[PCAP_ERRBUF_SIZE];
    struct pcap_pkthdr* hdr;
    const u_char* pkt_data;
    struct fileTs ft;
    bpf_program		filterCode;	// compiled filter code
    pcapDesc = new pcap_t*[nbFiles];

	for(int i=0; i<nbFiles; i++){
		if ((pcapDesc[i] = pcap_open_offline(files[i], errBuffer)) == NULL) {
			std::cerr << errBuffer << '\n';
			return -1;
		}
		if (pcap_compile(pcapDesc[i], &filterCode, code, 1, 0) < 0) {
			std::cerr <<  "parse code error\n";
			return -2;
		}
		if (pcap_setfilter(pcapDesc[i], &filterCode) < 0) {
			std::cerr << pcap_geterr(currPcapDesc) <<'\n';
			return -3;
		}
		initOffset = ftell(pcap_file(pcapDesc[i]));
		pcap_next_ex(pcapDesc[i],&hdr,&pkt_data);
		ft.pDesc = pcapDesc[i];
		ft.ts = hdr->ts.tv_sec;
		tmp.push_back(ft);

		if(strncmp(files[i],"tmp/VP",6)==0)	// Remove temp file once it'll be unused
			unlink(files[i]);
	}

    // sort regarding ts
    std::sort(tmp.begin(),tmp.end(),inf);
    for(int i=0; i<nbFiles; i++)
	        pcapDesc[i] = tmp[i].pDesc;
#endif
    return 0;
}

int Sniffer::loop(pcap_handler callback, u_char *userData,char noFile,int offset)
{
    for(fileInd=noFile; fileInd < nbFiles; fileInd++) {
	// Initialization of the offset's file
    #ifdef __MINGW32__
        bpf_program		filterCode;	// compiled filter code
        char errBuffer[PCAP_ERRBUF_SIZE];
        if ((currPcapDesc = pcap_open_offline(files[fileInd], errBuffer)) == NULL) {
			std::cerr << errBuffer << '\n';
			return -1;
		}
		if (pcap_compile(currPcapDesc, &filterCode, code, 1, 0) < 0) {
			std::cerr <<  "parse code error\n";
			return -2;
		}
		if (pcap_setfilter(currPcapDesc, &filterCode) < 0) {
			std::cerr << pcap_geterr(currPcapDesc) <<'\n';
			return -3;
		}    
    #else
    	currPcapDesc = pcapDesc[fileInd];
    	if(offset!=0){fseek(pcap_file(currPcapDesc),offset,SEEK_SET); offset=0;}
    	else fseek(pcap_file(currPcapDesc),initOffset,SEEK_SET);
    #endif
    pcap_loop(currPcapDesc, -1, callback, userData);
    #ifdef __MINGW32__
    pcap_close(currPcapDesc);
    #endif
    }
    return 1;
}

void Sniffer::breakloop(){
	pcap_breakloop(currPcapDesc);
}
